<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Maatwebsite\Excel\Facades\Excel;
use App\Imports\ProductsImport;
use App\Imports\MultiUnitsImport;
use Symfony\Component\HttpFoundation\Response;

class ImportController extends Controller
{
    public function importProduct(Request $request)
    {
        // Validate the file input
        $validator = Validator::make($request->all(), [
            'file' => 'required|file|mimes:xlsx,xls|max:2048', // Validate file type and size
        ]);

        if ($validator->fails()) {
            return response()->json([
                'success' => false,
                'message' => $validator->errors(), // Return validation errors
            ]);
        }

        try {
            $import = new ProductsImport;
            Excel::import($import, $request->file('file'));

            // Get the results of the import
            $results = $import->getResults();

            // Return success response with details of successful and failed rows
            return response()->json([
                'success' => true,
                'message' => 'Products imported successfully!',
                'successfulRows' => $results['successfulRows'],
                'failedRows' => $results['failedRows'],
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            // Handle exceptions and return error response
            return response()->json([
                'success' => false,
                'message' => 'Failed to import products: ' . $e->getMessage(),
            ], Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }

    public function importMultiUnit(Request $request)
    {
        // Validate the file input
        $validator = Validator::make($request->all(), [
            'file' => 'required|file|mimes:xlsx,xls|max:2048', // Validate file type and size
        ]);

        if ($validator->fails()) {
            return response()->json([
                'success' => false,
                'message' => 'Validation failed',
                'errors' => $validator->errors(), // Return validation errors
            ], Response::HTTP_BAD_REQUEST);
        }

        $successfulRows = [];

        try {
            // Import the data
            Excel::import(new MultiUnitsImport($successfulRows, $failedRows), $request->file('file'));

            // Return success response
            return response()->json([
                'success' => true,
                'message' => 'Multi units imported successfully!',
                'dataRows' => $successfulRows,
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            // Handle exceptions and return error response
            return response()->json([
                'success' => false,
                'message' => 'Failed to import multi units: ' . $e->getMessage(),
            ], Response::HTTP_INTERNAL_SERVER_ERROR);
        }
    }
}
