<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Artisan;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;

class MigrationController extends Controller
{
    public function runMigrations()
{
    $migratedTables = [];
    $failedMigrations = [];
    
    // Mengambil daftar file migrasi dari folder database/migrations
    $migrationFiles = glob(database_path('migrations') . '/*.php');
    
    foreach ($migrationFiles as $file) {
        $migrationName = basename($file, '.php');

        // Periksa apakah migrasi sudah diterapkan di tabel migrations
        if (DB::table('migrations')->where('migration', $migrationName)->exists()) {
            continue; // Lewati jika migrasi sudah diterapkan
        }

        try {
            // Jalankan migrasi
            Artisan::call('migrate', ['--path' => 'database/migrations/' . $migrationName . '.php']);
            $migratedTables[] = $migrationName;
        } catch (\Exception $e) {
            // Catat migrasi yang gagal dan pesan kesalahan
            $failedMigrations[] = [
                'migration' => $migrationName,
                'error' => $e->getMessage()
            ];
        }
    }
    
    return response()->json([
        'message' => 'Migrations run successfully',
        'migrated_tables' => $migratedTables,
        'failed_migrations' => $failedMigrations
    ]);
}

}
