<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Stocktransaction;
use App\Models\Substocktransaction;
use App\Models\Product;
use App\Models\Akun;
use App\Models\Credit;
use App\Models\Contact;
use App\Models\MultiUnit;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;



class StockController extends Controller
{
    public function getStockOutReport(Request $request){
        $data = Product::with('producttype','unit')
        ->whereHas('substocktransaction' , function($sub) use($request){
            $sub->whereHas('stocktransaction',function($stock) use($request){
                $stock = $stock->whereNotNull('cashin_id')->whereNull('pending')->whereNull('nonmoney')->whereNull('return');
                if (!empty($request->start_date) && !empty($request->end_date)) {
                    $request->start_date = date('Y-m-d',strtotime($request->start_date));
                    $request->end_date = date('Y-m-d',strtotime($request->end_date));
                    $stock = $stock->whereBetween('date',[$request->start_date,$request->end_date]);
                }else{
                    $stock = $stock->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
                }
                });
        });
        $data= $data->withSum(['substocktransaction'=>function($sub) use($request){
            $sub->whereHas('stocktransaction',function($stock) use($request){
                $stock = $stock->whereNotNull('cashin_id')->whereNull('pending')->whereNull('nonmoney')->whereNull('return');
                if (!empty($request->start_date) && !empty($request->end_date)) {
                    $request->start_date = date('Y-m-d',strtotime($request->start_date));
                    $request->end_date = date('Y-m-d',strtotime($request->end_date));
                    $stock = $stock->whereBetween('date',[$request->start_date,$request->end_date]);
                }else{
                    $stock = $stock->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
                }
            });
        }],'qty');
        
        $data =$data->withSum(['substocktransaction'=>function($sub) use($request){
            $sub->whereHas('stocktransaction',function($stock) use($request){
                $stock = $stock->whereNotNull('cashin_id')->whereNull('pending')->whereNull('nonmoney')->whereNull('return');
                if (!empty($request->start_date) && !empty($request->end_date)) {
                    $request->start_date = date('Y-m-d',strtotime($request->start_date));
                    $request->end_date = date('Y-m-d',strtotime($request->end_date));
                    $stock = $stock->whereBetween('date',[$request->start_date,$request->end_date]);
                }else{
                    $stock = $stock->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
                }
            });
        }],'total')->get();

        $response = [
            'success'=>true,
            'stocktransaction'=>$data,
        ];

        return response($response,200);
    }

    public function getStockTransaction(Request $request){
        $data = Stocktransaction::with('contact','cashin','cashout');

        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }
        // else{
        //     $data = $data->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
        // }

        $data = $data->get();

        $response = [
            'success'=>true,
            'stocktransaction'=>$data,
        ];

        return response($response,200);
    }

    public function getStockIn(Request $request){
        $type = isset($request->type) ? $request->type : null;
        $data = Stocktransaction::whereNotNull('cashout_id')->whereNull('pending')->whereNull('nonmoney')->whereNull('return');
        
        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }
        if($type === 'return'){
            $hasReturns = Substocktransaction::whereNotNull('return_trans_id')->pluck('return_trans_id');
            $data = $data->whereNotIn('id', $hasReturns);
        }
        // else{
        // $data = $data->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
        // }

        $data = $data->with('contact','cashout', 'substocktransaction.product.unit')->orderBy('created_at','DESC')->get();
        foreach ($data as $key => $value) {
            $day = date('Y-m-d',time());
            if ($value->payment_due>$day) {
                $value['due_date'] = Carbon::create($value->payment_due)->diffForHumans(null,true)." lagi";
            }else {
                $value['due_date'] = Carbon::create($value->payment_due)->diffForHumans(null,true)." yang lalu";
            }
        }
        
        $response = [
            'success'=>true,
            'stocktransaction'=>$data,

        ];

        return response($response,200);
    }

    public function getStockOut(Request $request){
        $type = isset($request->type) ? $request->type : null;
        $data = Stocktransaction::whereNotNull('cashin_id')->whereNull('pending')->whereNull('nonmoney')->whereNull('return');
        if (!empty($request->start_date) && !empty($request->end_date)) {
            $request->start_date = date('Y-m-d',strtotime($request->start_date));
            $request->end_date = date('Y-m-d',strtotime($request->end_date));
            $data = $data->whereBetween('date',[$request->start_date,$request->end_date]);
        }
        if($type === 'return'){
            // $hasReturns = Substocktransaction::whereNotNull('return_trans_id')->pluck('return_trans_id');
            // $data = $data->whereNotIn('id', $hasReturns);
        }
        // else{
        //     $data = $data->whereBetween('date',[date('Y-m-01',time()),date('Y-m-d',time())]);
        // }

        $data = $data->with('contact','cashin','credit', 'substocktransaction.product.unit')->orderBy('created_at','DESC')->get();

        $response = [
            'success'=>true,
            'stocktransaction'=>$data,
        ];

        return response($response,200);
    }


    public function getStockTransactionDetail(Request $request){
        $data = Stocktransaction::where('id',$request->id)->with('contact','cashin','cashout','substocktransaction','substocktransaction.product.unit','credit','credit.cashout','credit.cashin')->get();
        $data = $data->map(function($item){
            $sum_qty = $item->substocktransaction->map(function($item2){
                return array_merge($item2->toArray(), ['qty' => $item2->qty]);
            });
            return array_merge($item->toArray(), ['substocktransaction' => $sum_qty]);
        });
        $response = [
            'success'=>true,
            'stocktransaction'=>$data
        ];
        
        return response($response,200);
    }

    public function createStockIn(Request $request){

        $request->validate([
            'contact_id' =>'required',
            'cashout_id' =>'nullable',
            'staff' =>'required',
            'paid' =>'required',
            'discount' =>'nullable',
            'payment_due' =>'required',
            'date' =>'required',

            'product_id.*' =>'required',
            'qty.*'  =>'required',
            'purchase_price.*'  =>'required',
            'purchase_satuan.*'  =>'nullable',
            'purchase_multi_satuan.*'  =>'nullable',
            'unit_id.*'  =>'nullable',
            'total.*'  =>'required|numeric',
        ]); 
        $paid=0;
        if (!empty($request->paid) || $request->paid>0) {
            $request->validate([
                'cashout_id' =>'required',
            ]); 
        }
        if (empty($request->cashout_id)) {
            $request->cashout_id = Akun::where('iscash',true)->first()->id;
        }

        $stock = new Stocktransaction;  
        $stock->contact_id = $request->contact_id;
        $stock->cashout_id = $request->cashout_id;
        $stock->staff = $request->staff;
        $stock->description = $request->desc;
        $stock->pending = null;
        $stock->date = date("Y-m-d", strtotime($request->date));
        $stock->paid = $request->paid;
        $stock->payment_due = date("Y-m-d", strtotime($request->payment_due));
        $stock->save();

        $data = $request->product_id;
        $total = 0;
        foreach ( $data as $key => $value) {
            $total = $total + $request->total[$key];
            // jika terjadi masalah di $total
            if ($total <= 0) {
                return response()->json(['message' => 'Total cannot be zero or negative.'], 400);
            }

            $sub = new Substocktransaction;
            $sub->stocktransaction_id = $stock->id;
            $sub->product_id = $request->product_id[$key];
            $sub->qty = $request->qty[$key];
            $sub->purchase_price = $request->purchase_price[$key];
            $sub->left = $request->qty[$key];
            $sub->total = $request->total[$key];
            $sub->save();

            $substocktransaction[]= $sub;

            $product = Product::find($sub->product_id);
            $product->qty = $product->qty + $sub->qty;
            $product->save();
        }

        $akun = Akun::find($request->cashout_id);
        $akun->total = $akun->total - $stock->paid;
        $akun->save();

        $akun = Akun::where('name','=','Persediaan Barang')->first();
        $akun = Akun::find($akun->id);
        $akun->total = $akun->total + $total;
        $akun->save();

        $credit = new Credit;
        $credit->stocktransaction_id = $stock->id;
        $credit->cashout_id = $stock->cashout_id;
        $credit->total = $stock->paid;
        $credit->save();

        $paid=0;
        if (!empty($request->id)) {
            $this->pendingToIn($request);
            $pending = Stocktransaction::find($request->id);
            $paid=$pending->paid;
            
            $credit = new Credit;
            $credit->stocktransaction_id = $stock->id;
            $credit->cashout_id = $pending->cashout_id;
            $credit->total = $pending->paid;
            $credit->save();
        }
        
        $akun = Akun::where('name','=','Hutang Pembelian Non Tunai')->first();
        $akun = Akun::find($akun->id);
        $akun->total = $akun->total + (($total - $request->discount) - $stock->paid);
        $akun->save();

        $akun = Akun::where('name','=','Potongan Pembelian')->first();
        $akun = Akun::find($akun->id);
        $akun->total = $akun->total + $request->discount;
        $akun->save();
        
        $stock = Stocktransaction::find($stock->id);
        $stock->discount = $stock->discount + $request->discount;
        $stock->total = $total;
        $stock->paid = $stock->paid+$paid;
        $stock->save();

        foreach ($data as $key => $value) {
            $sub = new Substocktransaction;
            $sub->product_id = $request->product_id[$key];
        
            $product = Product::find($sub->product_id);
        
            $multis = MultiUnit::with('multi_unit')
                            ->where('product_id', $sub->product_id)
                            ->get();

            if ($request->purchase_satuan[$key] != 0) {
                $product->purchase_price = $request->purchase_satuan[$key];
        
                if ($product->profit_strategy === 'persentase') {
                    $percent = $product->profit_strategy_percent;
        
                    $product->selling_price = $product->purchase_price + ($product->purchase_price * ($percent / 100));
                    $product->selling_price = round($product->selling_price, -2);
                }

                if ($multis->isNotEmpty()) {
                    foreach ($multis as $m) {
                        $m->multi_purchase_price = $product->purchase_price * $m->multi_qty;
                        
                        if ($product->profit_strategy === 'persentase') {
                            $m->multi_selling_price = $m->multi_purchase_price + ($m->multi_purchase_price * ($percent / 100));
                            $m->multi_selling_price = round($m->multi_selling_price, -2);
                        }
        
                        $m->save();
                    }
                }
            }

            if ($request->purchase_multi_satuan[$key] != 0) {
                if ($request->unit_id[$key] != 'none') {
                    $multi = MultiUnit::where('product_id', $product->id)
                                    ->where('multi_unit', $request->unit_id[$key])
                                    ->first();
                    if ($multi) {
                        $multi->multi_purchase_price = $request->purchase_multi_satuan[$key];

                        if ($product->profit_strategy === 'persentase') {
                            $percent = $product->profit_strategy_percent;

                            $multi->multi_selling_price = $multi->multi_purchase_price + ($multi->multi_purchase_price * ($percent / 100));
                            $multi->multi_selling_price = round($multi->multi_selling_price, -2);
                        }

                        $multi->save();

                        if ($multi->multi_qty != 0) { 
                            $mq = $multi->multi_purchase_price / $multi->multi_qty;
                            $mq = round($mq, -2);

                            $product->purchase_price = $mq;

                            if ($product->profit_strategy === 'persentase') {
                                $product->selling_price = $product->purchase_price + ($product->purchase_price * ($percent / 100));
                                $product->selling_price = round($product->selling_price, -2);
                            }
                        }

                        if ($multis->isNotEmpty()) {
                            foreach ($multis as $m) {
                                if ($m->id !== $multi->id) {
                                    $m->multi_purchase_price = $product->purchase_price * $m->multi_qty;

                                    if ($product->profit_strategy === 'persentase') {
                                        $m->multi_selling_price = $m->multi_purchase_price + ($m->multi_purchase_price * ($percent / 100));
                                        $m->multi_selling_price = round($m->multi_selling_price, -2);
                                    }

                                    $m->save();
                                }
                            }
                        }
                    }
                }
            }

            $product->save();
        }        

        $response = [
            'success'=>true,
            'stockktransaction'=>$stock,
            'substocktransaction'=>$substocktransaction,
            'product'=>$product,
            'multis'=>$multis,
        ];

        return response($response,200);
    }

    public function createStockOut(Request $request){

        $request->validate([
            'contact_id' =>'required',
            'cashin_id' =>'nullable',
            'staff' =>'required',
            'paid' =>'required',
            'date' =>'required',
            'payment_due' =>'required',
            'discount' =>'nullable',
            'cash_received' => 'nullable',
            'cash_change' => 'nullable',

            'product_id.*' =>'required',
            'selling_price.*' =>'required',
            'qty.*'  =>'required',
            'total.*'  =>'required|numeric',
        ]); 
        if (!empty($request->paid) || $request->paid>0) {
            $request->validate([
                'cashin_id' =>'required',
            ]); 
        }
        if (empty($request->cashin_id)) {
            $request->cashin_id = Akun::where('iscash',true)->first()->id;
        }

        $contact = Contact::where('id',$request->contact_id)->first();
        $sum = 0;

        foreach ( $request->total as $key => $value) {
            $sum = $sum + $request->total[$key];
        }
        $paid=0;
        if (!empty($request->id)) {
            $pending = Stocktransaction::find($request->id);
            $paid=$pending->paid;
        }
        $request->paid=$request->paid+$paid;
        $hutang = ($sum - $request->discount) - $request->paid ;
        if ($hutang > $contact->type()->first()->maxdebt) {
            return response(['error'=>'Hutang melebihi batas'],400);
        }
        $request->paid=$request->paid-$paid;

        if (!empty($request->id)) {
            $this->pendingToOut($request);
        }
        $paydue = date("Y-m-d", strtotime($request->payment_due));
        $day = $contact->type()->first()->max_paydue;
        $max_patdue=date('Y-m-d',time()+(60*60*24*$day));
        if ($paydue > $max_patdue && $contact->type()->first()->max_paydue!=null) {
            return response(['error'=>'Jatuh tempo melebihi batas'],400);
        }
        
        $stock = new Stocktransaction;
        $stock->contact_id = $request->contact_id;
        $stock->cashin_id = $request->cashin_id;
        $stock->description = $request->desc;
        $stock->staff = $request->staff;
        $stock->pending = null;
        $stock->date = date("Y-m-d", strtotime($request->date));
        $stock->paid = $request->paid;
        $stock->payment_due = date("Y-m-d", strtotime($request->payment_due));
        $stock->cash_received = $request->cash_received;
        $stock->cash_change = $request->cash_change;
        $stock->save();

        $data = $request->product_id;
        $total = 0;
        $jasa = 0;
        $totalhpp=0;
        $lasthb=0;
        $qty=0;
        foreach ( $data as $key => $value) {
            $sub = new Substocktransaction;
            $sub->stocktransaction_id = $stock->id;
            $sub->product_id = $request->product_id[$key];
            $sub->selling_price = $request->selling_price[$key];
            $sub->qty = $request->qty[$key];
            $sub->total = $request->total[$key];
            $sub->save();

            $substocktransaction[]= $sub;

            $product = Product::find($sub->product_id);
            if ($product->category=='service') {

                $akun = Akun::where('name','=','Pendapatan Jasa')->first();
                $akun = Akun::find($akun->id);
                $akun->total = $akun->total + $sub->total;
                $akun->save();
                $jasa = $jasa + $sub->total;
                $total = $total + $sub->total;
                continue;
            }
            $product->qty = $product->qty - $sub->qty;
            $product->save();
            
            $qty = $sub->qty;
            $subin = Substocktransaction::where('left','>',0)
            ->whereHas('stocktransaction',function($query){
                $query->whereNull('pending');
            })->where('product_id','=',$sub->product_id)->get();
            
            // temp array substock ids
            $substocktransactions_ids = [];
            
            foreach ($subin as $key => $value) {

                // push id
                array_push($substocktransactions_ids, $value->id);

                if ($qty <= $value->left) {

                    $set = $value->left-$qty;
                    
                    $sibin = Substocktransaction::find($value->id);
                    $sibin->left = $set;
                    $sibin->save();

                    $totalhpp = $totalhpp +($value->purchase_price * $qty);
                    break;
                }else{
                    $set = 0;
                    $qty = $qty - $value->left;
                    $totalhpp = $totalhpp + ($value->purchase_price * $value->left);

                    $sibin = Substocktransaction::find($value->id);
                    $sibin->left = $set;
                    $sibin->save();
                }
                
                $lasthb=$value->purchase_price;
            }
            
            $sibin = Substocktransaction::find($sub->id);
            $sibin->hpp = $totalhpp;
            $sibin->substocktransactions_ids = join(",", $substocktransactions_ids); // merge array with string ","
            $sibin->save();
            $totalhpp=0;
            $total = $total + $sub->total;
            
        }
              




        
        $credit = new Credit;
        $credit->stocktransaction_id = $stock->id;
        $credit->cashin_id = $stock->cashin_id;
        $credit->total = $stock->paid;
        $credit->save();
        
        $paid=0;
        if (!empty($request->id)) {
            $pending = Stocktransaction::find($request->id);
            $paid=$pending->paid;

            $credit = new Credit;
            $credit->stocktransaction_id = $stock->id;
            $credit->cashin_id = $pending->cashin_id;
            $credit->total = $pending->paid;
            $credit->save();
        }

        $stock = Stocktransaction::find($stock->id);
        $stock->discount = $stock->discount + $request->discount;
        $stock->total = $total;
        $stock->paid = $stock->paid+$paid;
        $stock->save();

        $response = [
            'success'=>true,
            'stockktransaction'=>$stock,
            'substocktransaction'=>$substocktransaction,
        ];
        return response($response,200);
    }

    public function pendingToIn(Request $request){
        $stock = Stocktransaction::find($request->id);
        $stock->pending = false;
        $stock->save();

        $akun = Akun::where('name','=','Uang Muka Pesanan Pembelian')->first();
        $akun = Akun::find($akun->id);
        $akun->total = $akun->total -  $stock->paid;
        $akun->save();

        $akun = Akun::find($request->cashout_id);
        $akun->total = $akun->total + $stock->paid;
        $akun->save();
    }

    public function pendingToOut(Request $request){
        $stock = Stocktransaction::find($request->id);
        $stock->pending = false;
        $stock->save();

        $akun = Akun::where('name','=','Hutang Pesanan Penjualan')->first();
        $akun = Akun::find($akun->id);
        $akun->total = $akun->total - $stock->paid;
        $akun->save();
        
        $akun = Akun::find($request->cashin_id);
        $akun->total = $akun->total - $stock->paid;
        $akun->save();
    }

    public function deleteStockTransaction(Request $request){
        // data yang ingin di hapus
        $stock = Stocktransaction::with('substocktransaction')
                                  ->find($request->id);
        // return response($stock, 200);
        $sids = $stock->substocktransaction->map(function ($item) {
            return $item->id;
        });
        $hasReturn = Substocktransaction::whereIn('return_trans_id', $sids)
                                    ->with('stocktransaction')
                                    ->get()
                                    ->map(function ($item) {
                                        if($item->stocktransaction->return === 'out'){
                                            return 'RETUR PEMBELIAN no. ' . $item->stocktransaction->id;
                                        }
                                        else if($item->stocktransaction->return === 'in'){
                                            return 'RETUR PENJUALAN no. ' . $item->stocktransaction->id;
                                        }
                                    });    

        if(count($hasReturn) > 0){
            $response = [
                'success'=> false,
                'message'=> "Produk dalam transaksi ini telah diretur, " . $hasReturn[0] . ". Jika Anda ingin menghapus ini, silakan hapus transaksi tersebut terlebih dahulu.",
            ];
            
            return response($response,400);
        }

        if($stock){
            // mengambil semua transaksi pembelian sebelumnya
            //  $out = Stocktransaction::where('id', $stock->id)
            //                ->with('substocktransaction')
            //                ->get()
            //                ->map(function ($item){
            //                  $substocktransaction = $item->substocktransaction;
            //                 //  return $substocktransaction->map(function ($item2) {
            //                     return $substocktransaction->flatMap(function ($item2) {
            //                      return Substocktransaction::where('substocktransactions_ids', 'LIKE', '%' . $item2->id . '%')->first();
            //                  });
            //                     // $substocktransaction = $item->substocktransaction;
            //                     // // Menggunakan flatMap untuk menyatukan hasil setiap item
            //                     // return $substocktransaction->flatMap(function ($item2) {
            //                     //     // Melakukan pencarian untuk setiap item2
            //                     //     return Substocktransaction::where('substocktransactions_ids', 'LIKE', '%' . $item2->id . '%')->get();
            //                     // });
            //                });
            //  $out = $out[0][0];
            $out = Stocktransaction::where('id', $stock->id)
                       ->with('substocktransaction')
                       ->get()
                       ->map(function ($item){
                           $substocktransaction = $item->substocktransaction;
                           // Menggunakan flatMap untuk menyatukan hasil setiap item
                           return $substocktransaction->flatMap(function ($item2) {
                               // Melakukan pencarian untuk setiap item2
                               return Substocktransaction::where('substocktransactions_ids', 'LIKE', '%' . $item2->id . '%')->get();
                           });
                       })
                       ->flatten(); // Menggabungkan semua hasil menjadi satu koleksi

            $out = $out->first();
    
             if($stock->cashin_id){
                 $akun = Akun::find($stock->cashin_id);
                 $akun->total = $akun->total - $stock->paid;
                 $akun->save();
    
                 $akun = Akun::where('name','=','Piutang Penjualan')->first();
                 $akun = Akun::find($akun->id);
                 $akun->total = $akun->total - ($stock->total -$stock->discount - $stock->paid);
                 $akun->save();
    
                 $sub =Substocktransaction::where('stocktransaction_id','=',$stock->id)->get();
                 $totalhpp=0;
                 $jasa=0;
                 foreach ($sub as $key => $value) {
                     $product = Product::find($value->product_id);
                     if ($product->category=='service') {
    
                         $akun = Akun::where('name','=','Pendapatan Jasa')->first();
                         $akun = Akun::find($akun->id);
                         $akun->total = $akun->total - $value->total;
                         $akun->save();
                         $jasa = $jasa + $value->total;
                         continue;
                     }
                     $product->qty = $product->qty + $value->qty;
                     $product->save(); 
    
                     $totalhpp = $totalhpp + $value->hpp;
                     $qty = $value->qty;
                     $subin = explode(",", $value->substocktransactions_ids);
                     foreach ($subin as $key => $value) {
                        $sibin = Substocktransaction::find($value);
                        
                        if ($qty <= $sibin->qty) {
                            $set = $sibin->left + $qty;
                            $sibin->left = $set;
                            $sibin->save();
                        }else{
                            $set = $sibin->qty;
                            $qty = $qty - $sibin->qty;
                            $sibin->left = $set;
                            $sibin->save();
                        }
                     }
    
                 }
    
                 $akun = Akun::where('name','=','Persediaan Barang')->first();
                 $akun = Akun::find($akun->id);
                 $akun->total = $akun->total + $totalhpp;
                 $akun->save();
    
                 $akun = Akun::where('name','=','Pendapatan Penjualan')->first();
                 $akun = Akun::find($akun->id);
                 $akun->total = $akun->total - ($stock->total-$jasa);
                 $akun->save();
    
                 $akun = Akun::where('name','=','Harga Pokok Penjualan')->first();
                 $akun = Akun::find($akun->id);
                 $akun->total = $akun->total - $totalhpp;
                 $akun->save();
    
                 $akun = Akun::where('name','=','Potongan Penjualan')->first();
                 $akun = Akun::find($akun->id);
                 $akun->total = $akun->total - $stock->discount;
                 $akun->save();
    
                 Substocktransaction::where('stocktransaction_id','=',$stock->id)->delete();
                 Credit::where('stocktransaction_id','=',$stock->id)->delete();

                 $stock->delete();
                 $stock->credit()->delete();

                 $response = [
                     'success'=>true,
                     'stocktransaction'=> $stock,
                 ];
                 return response($response,200);
                
             }elseif ($stock->cashout_id) {
                 if(!$out){
                     $akun = Akun::find($stock->cashout_id);
                     $akun->total = $akun->total + $stock->total;
                     $akun->save();
        
                     $totalhpp=0;
                     $sub =Substocktransaction::where('stocktransaction_id','=',$stock->id)->get();
                     foreach ($sub as $key => $value) {
                         $product = Product::find($value->product_id);
                         $product->qty = $product->qty - $value->qty;
                         $product->save(); 
        
                         $totalhpp = $totalhpp + ($value->left * $value->purchase_price);
                         //kembalikan qty jika return penjualan
                         if($stock->return === 'in'){
                            $s_r = Substocktransaction::where('stocktransaction_id', $value->return_trans_id)->where('product_id', $value->product_id)->first();
                            if($s_r){
                                $s_r->qty = $s_r->qty + $value->qty;
                                $s_r->save();
                            }
                         }
                     }
        
                     $akun = Akun::where('name','=','Persediaan Barang')->first();
                     $akun = Akun::find($akun->id);
                     $akun->total = $akun->total - $stock->total;
                     $akun->save();
        
                     $akun = Akun::where('name','=','Potongan Pembelian')->first();
                     $akun = Akun::find($akun->id);
                     $akun->total = $akun->total - $stock->discount;
                     $akun->save();
        
                     $akun = Akun::where('name','=','Hutang Pembelian Non Tunai')->first();
                     $akun = Akun::find($akun->id);
                     $akun->total = $akun->total - (($totalhpp - $stock->discount) - $stock->paid);
                     $akun->save();
        
                     Substocktransaction::where('stocktransaction_id','=',$stock->id)->delete();
                     $stock->delete();
                     $stock->credit()->delete();
    
                     //  $response = [
                     //      'success'=>true,
                     //      'message'  => 'boleh',
                     //      'buy' => $total_buy,
                     //      'sell' => $total_sell
                     //  ];
                     //  return response($response,200);
    
                     $response = [
                         'success'=>true,
                         'stocktransaction'=> $stock,
                     ];
                     return response($response,200);
                 }else{
                     $response = [
                         'success'=>false,
                         'message' => 'Produk dalam transaksi ini telah dikeluarkan dalam PENJUALAN / ITEM KELUAR dengan nomor transaksi: ' . $out->stocktransaction_id . '. Jika Anda ingin menghapus transaksi ini, silakan hapus transaksi tersebut terlebih dahulu.',
                     ];
                     return response($response,400);
                 }
             }

         } else {
             $response = [
                 'success'=>false,
                 'message'=> 'not found' 
             ];
             return response($response,400);
         }
    }
}